/**********************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: PropertyListCommand.java,v 1.6 2006-04-06 15:28:00 samwai Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.internal.execution.local.common;

import java.util.Vector;

public class PropertyListCommand extends CommandElement implements Constants {
	private int size = 0;
	private Vector names = new Vector();
	private Vector types = new Vector();
	private Vector values = new Vector();

	public PropertyListCommand() {
		super();
		_tag = RA_PROPERTY_LIST;
	}

	public int getPropertyListLength() {
		return size;
	}

	public SetNVPairCommand[] getPropertyListValues() {
		SetNVPairCommand[] results = new SetNVPairCommand[size];
		for(int i = 0; i < size; i++) {
			results[i] = new SetNVPairCommand();
			results[i].setName(((RAString)names.elementAt(i)).getData());
			results[i].setType(((RAString)types.elementAt(i)).getData());
			results[i].setValue(((RAString)values.elementAt(i)).getData());
		}
		return results;
	}

	public void addPropertyListValue(String name, String type, String value) {
		RAString rName = new RAString(name);
		RAString rType = new RAString(type);
		RAString rValue = new RAString(value);

		names.add(rName);
		types.add(rType);
		values.add(rValue);

		/*
		 * Bug 116585: PropertyListCommand does not work for IAC
		 */
		size++;
	}

	public int readFromBuffer(byte[] buffer, int offset) {
		int current = offset;

		current = super.readFromBuffer(buffer, current);

		// Read the number of entries
		size = (int)Message.readRALongFromBuffer(buffer, current);
		current += sizeofLong;

		// Read the name, type, value entries
		for(int i = 0; i < size; i++) {
			RAString name = new RAString("");
			RAString type = new RAString("");
			RAString value = new RAString("");
			current = Message.readRAStringFromBuffer(buffer, current, name);
			current = Message.readRAStringFromBuffer(buffer, current, type);
			current = Message.readRAStringFromBuffer(buffer, current, value);
			names.addElement(name);
			types.addElement(type);
			values.addElement(value);
		}

		return current;
	}

	public int writeToBuffer(byte[] buffer, int offset) {
		int current = offset;

		current = super.writeToBuffer(buffer, current);

		/* Names */
		current = Message.writeRALongToBuffer(buffer, current, size);
		for(int i = 0; i < size; i++) {
			current = Message.writeRAStringToBuffer(buffer, current, ((RAString)names.elementAt(i)));
			current = Message.writeRAStringToBuffer(buffer, current, ((RAString)types.elementAt(i)));
			current = Message.writeRAStringToBuffer(buffer, current, ((RAString)values.elementAt(i)));
		}

		return current;
	}

	public int getSize() {
		int size = super.getSize();

		size += sizeofLong; // entry size
		for(int i = 0; i < size; i++) {
			size += ((RAString)names.elementAt(i)).getSize();
			size += ((RAString)types.elementAt(i)).getSize();
			size += ((RAString)values.elementAt(i)).getSize();
		}

		return size;
	}
}
