/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TreePart.java,v 1.4 2005-08-19 21:18:16 hleung Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.editor.form.base;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.FormToolkit;

public class TreePart extends StructuredViewerPart
{

	/**
	 * Constructor for TreePart.
	 * @param buttonLabels
	 */
	public TreePart(String[] buttonLabels)
	{
		super(buttonLabels);
	}
    /**
     * Constructor for TreePart.
     * @param buttonLabels
     * @param toolTips
     */
    public TreePart(String[] buttonLabels,String [] toolTips)
    {
        super(buttonLabels,toolTips);
    }

	protected StructuredViewer createStructuredViewer(Composite parent, int style, FormWidgetFactory factory)
	{
		return  createStructuredViewer(parent, style, (Object)factory);
	}
	
	
	/**
	 * @see StructuredViewerPart#createStructuredViewer(Composite, FormToolkit)
	 */
	protected StructuredViewer createStructuredViewer(Composite parent, int style, FormToolkit factory)
	{
		return  createStructuredViewer(parent, style, (Object)factory);
	}
	
	
	/**
	 * A helper method to avoid repeated statments.  Transfer content and take this method out once the 
	 * deprecated version of createMainControl is taken out.
	 */	
	private StructuredViewer createStructuredViewer(Composite parent, int style, Object factory)
	{
		style |= SWT.H_SCROLL | SWT.V_SCROLL;
		if (factory == null)
		{
			style |= SWT.BORDER;
		}
		else
		{
			style |= FormWidgetFactory.BORDER_STYLE;
		}
		TreeViewer treeViewer = new TreeViewer(parent, style);
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener()
		{
			public void selectionChanged(SelectionChangedEvent e)
			{
				TreePart.this.selectionChanged((IStructuredSelection)e.getSelection());
			}
		});
		treeViewer.addDoubleClickListener(new IDoubleClickListener()
		{
			public void doubleClick(DoubleClickEvent e)
			{
				TreePart.this.handleDoubleClick((IStructuredSelection)e.getSelection());
			}
		});
		return treeViewer;
	}

	public TreeViewer getTreeViewer()
	{
		return (TreeViewer)getViewer();
	}
	
	/*
	 * @see SharedPartWithButtons#buttonSelected(int)
	 */
	protected void buttonSelected(Button button, int index)
	{
	}

	protected void selectionChanged(IStructuredSelection selection)
	{
	}
	
	protected void handleDoubleClick(IStructuredSelection selection)
	{
	}
}