/**********************************************************************
 * Copyright (c) 2005, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TraceInteractions.java,v 1.9 2005-10-17 16:43:00 sduguet Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.uml2sd.trace.loaders;

import java.util.ArrayList;
import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.extensions.SimpleSearchQuery;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCMethodInvocation;
import org.eclipse.hyades.models.trace.TRCProcess;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.trace.views.internal.TraceUIMessages;
import org.eclipse.hyades.trace.views.internal.TraceUIPlugin;
import org.eclipse.hyades.ui.filters.FilterActions;
import org.eclipse.hyades.ui.filters.IFilterQueryProvider;
import org.eclipse.hyades.ui.filters.IFilterViewer;
import org.eclipse.hyades.uml2sd.trace.TraceSDUtil;
import org.eclipse.hyades.uml2sd.trace.actions.internal.ChangeLifelineLoaderAction;
import org.eclipse.hyades.uml2sd.trace.actions.internal.UndoDrillDownMenuAction;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.FilterQuery;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.FoundInPage;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.ITraceMessage;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceCallStack;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceCollapsableSyncMessage;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceCollapsableSyncMessageReturn;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceCollapsedLifelines;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceCollapsedSyncMessage;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceCollapsedSyncMessageReturn;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceInteractionUpdate;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceInteractionUtils;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceLifeline;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceLifelineDraft;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceMessageAndCallAssociation;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TracePage;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceProcess;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceRootProperty;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceSyncMessage;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceSyncMessageReturn;
import org.eclipse.hyades.uml2sd.trace.preferences.ITraceInteractionPreferenceListener;
import org.eclipse.hyades.uml2sd.trace.selection.IDateSelection;
import org.eclipse.hyades.uml2sd.trace.selection.IEObjectSelection;
import org.eclipse.hyades.uml2sd.trace.util.TIUtils;
import org.eclipse.hyades.uml2sd.ui.actions.provider.IExtendedFilterProvider;
import org.eclipse.hyades.uml2sd.ui.actions.provider.ISDAdvancedPagingProvider;
import org.eclipse.hyades.uml2sd.ui.actions.provider.ISDCollapseProvider;
import org.eclipse.hyades.uml2sd.ui.actions.provider.ISDFilterProvider;
import org.eclipse.hyades.uml2sd.ui.actions.provider.ISDGraphNodeSupporter;
import org.eclipse.hyades.uml2sd.ui.actions.provider.ISDInternalMesFilterProvider;
import org.eclipse.hyades.uml2sd.ui.actions.provider.ISDPropertiesProvider;
import org.eclipse.hyades.uml2sd.ui.actions.widgets.Criteria;
import org.eclipse.hyades.uml2sd.ui.actions.widgets.FilterCriteria;
import org.eclipse.hyades.uml2sd.ui.actions.widgets.FilterListDialog;
import org.eclipse.hyades.uml2sd.ui.core.BaseMessage;
import org.eclipse.hyades.uml2sd.ui.core.Frame;
import org.eclipse.hyades.uml2sd.ui.core.GraphNode;
import org.eclipse.hyades.uml2sd.ui.core.ITimeRange;
import org.eclipse.hyades.uml2sd.ui.core.Lifeline;
import org.eclipse.hyades.uml2sd.ui.core.LifelineCategories;
import org.eclipse.hyades.uml2sd.ui.drawings.impl.ImageImpl;
import org.eclipse.hyades.uml2sd.ui.load.LoadersManager;
import org.eclipse.hyades.uml2sd.util.SDUtil;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.DialogSettings;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;

/**
 * The abstract trace loader.
 * It is based on the parsing of MethodInvocations from the model.
 * Lifelines attribution is made by the concrete classes.
 */
public class TraceInteractions extends BaseTraceInteractions
	implements IExtendedFilterProvider, ISDAdvancedPagingProvider,
			   ISDInternalMesFilterProvider, ISDCollapseProvider,
			   ITraceInteractionPreferenceListener, ISDPropertiesProvider, ISDFilterProvider, IFilterViewer {

	/**
	 * Categories:
	 * - first one for classes
	 * - second one for instances of classes
	 * - third one for collapsed classes and/or objects
	 */
	public static final int CATEGORY_CLASSES = 0;
	public static final int CATEGORY_OBJECTS = 1;
	public static final int CATEGORY_NODE = 2;
	public static final int CATEGORY_AGENT = 3;
	public static final int CATEGORY_PROCESS = 4;
	public static final int CATEGORY_THREAD = 5;
	public static final int CATEGORY_COLLAPSED = 6;
	public static final int CATEGORY_CARD = 7;

	private static LifelineCategories[] traceCategories;
	static {
		traceCategories = new LifelineCategories[CATEGORY_CARD];
		traceCategories[CATEGORY_CLASSES] = new LifelineCategories();
		traceCategories[CATEGORY_CLASSES].setName(TraceUIMessages._143); //$NON-NLS-1$
		traceCategories[CATEGORY_CLASSES].setImage(new ImageImpl(TraceSDUtil.getResourceImage("full/obj16/class.gif"))); //$NON-NLS-1$
		traceCategories[CATEGORY_OBJECTS] = new LifelineCategories();
		traceCategories[CATEGORY_OBJECTS].setName(TraceUIMessages._144); //$NON-NLS-1$
		traceCategories[CATEGORY_OBJECTS].setImage(new ImageImpl(TraceSDUtil.getResourceImage("full/obj16/classobject_obj.gif"))); //$NON-NLS-1$
		traceCategories[CATEGORY_NODE] = new LifelineCategories();
		traceCategories[CATEGORY_NODE].setName(TraceUIMessages._139); //$NON-NLS-1$
		traceCategories[CATEGORY_NODE].setImage(new ImageImpl(TraceSDUtil.getResourceImage("full/obj16/node_obj.gif"))); //$NON-NLS-1$
		traceCategories[CATEGORY_AGENT] = new LifelineCategories();
		traceCategories[CATEGORY_AGENT].setName(TraceUIMessages._141); //$NON-NLS-1$
		traceCategories[CATEGORY_AGENT].setImage(new ImageImpl(TraceSDUtil.getResourceImage("full/obj16/agent_obj.gif"))); //$NON-NLS-1$
		traceCategories[CATEGORY_PROCESS] = new LifelineCategories();
		traceCategories[CATEGORY_PROCESS].setName(TraceUIMessages._140); //$NON-NLS-1$
		traceCategories[CATEGORY_PROCESS].setImage(new ImageImpl(TraceSDUtil.getResourceImage("full/obj16/process_obj.gif"))); //$NON-NLS-1$
		traceCategories[CATEGORY_THREAD] = new LifelineCategories();
		traceCategories[CATEGORY_THREAD].setName(TraceUIMessages._142); //$NON-NLS-1$
		traceCategories[CATEGORY_THREAD].setImage(new ImageImpl(TraceSDUtil.getResourceImage("full/obj16/thread_obj.gif"))); //$NON-NLS-1$
		traceCategories[CATEGORY_COLLAPSED] = new LifelineCategories();
		traceCategories[CATEGORY_COLLAPSED].setName(TraceUIMessages._152); //$NON-NLS-1$
		traceCategories[CATEGORY_COLLAPSED].setImage(new ImageImpl(TraceSDUtil.getResourceImage("full/obj16/collapsed_obj.gif"))); //$NON-NLS-1$
	}
	
	/**
	 * Internal message filtering:
	 * internal means that the message destination is the same as the origin (same lifeline)
	 * see also ISDInternalMesFilterProvider
	 */
	private boolean internalMessagesFiltered = false;
	
	private SimpleSearchQuery filter;
	private IFilterQueryProvider filterQueryProvider;
	
	/**
	 * Paging related attributes
	 */
	private int maximumMessagesByPage = (int)TraceUIPlugin.getDefault().getPreferenceStore().
		getLong(ITraceInteractionPreferenceListener.PAGE_SIZE);
	private int nbMessages, nbMessagesInThisPage, pageNumber = 0;
	private boolean hasNextPage = false;
	
	/**
	 * Implementation of IUml2SDLoader
	 */
	public void onSetViewer() {
		view.setExtendedFilterProvider(this);
		view.setSDFilterProvider(this);
		view.setSDPagingProvider(this);
		view.setInternalMessageFilterProvider(this);
		view.setCollapsingProvider(this);
		view.getSDWidget().setReorderMode(true);
		view.setSDPropertiesProvider(this);
		currentFilters = FilterListDialog.getGlobalFilters();
		updateHasPositiveLifelineFilters();
		collapsedLifelines = loadCollapsedLifelines();
		collapsedMessages = loadCollapsedMessages();
	}
	
	/**
	 * 
	 */
	public void clearInternals() {
		super.clearInternals();
		pageNumber = 0;
		tracePages = null;
//		selectedThreadList = null;
		lifelineReordered = null;
	}
	
	/**
	 * @return
	 */
	protected TraceInteractionUpdate createTraceInteractionUpdate() {
		TraceInteractionUpdate ret = new TraceInteractionUpdate();
//		if (selectedThreadList != null) {
//			ret.setSelectedThreadList(selectedThreadList);
//		}
		return ret;
	}

	/**
	 * Let's parse the model for the translation
	 * @param eObject
	 */
	protected synchronized void parseModel(List<TRCAgentProxy> list) {
		long start = 0;
		if (TraceSDUtil.debugEvents) {
			start = System.currentTimeMillis();
		}
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("Start parseModel()"); //$NON-NLS-1$
		}
		clearInternals();
		currentMofObjectsFromPDProjectExplorer = list;
		createFrame();
		if (isEmpty()) {
			if (view != null && !view.getSDWidget().isDisposed()) {
				setFrameName();
				view.setFrameSync(frame);
			}
			if (TraceSDUtil.debugEvents) {
				TraceSDUtil.debugUml2SDTraceEvents("Abort parseModel()"); //$NON-NLS-1$
			}
			return;
		}
		try {
			computeModel();
			fillPage(new TraceInteractionUpdate());
			if (view != null && !view.getSDWidget().isDisposed() && frame != null && !isCurrentLoadCanceled()) {
				setFrameName();
				view.setFrameSync(frame);
			}
		} catch (ConcurrentModificationException e) {
			// Give up silently while the model is being alimented concurrently
		} catch (Throwable e) {
			e.printStackTrace();
		}
		if (TraceSDUtil.debugEvents) {
			if (isCurrentLoadCanceled()) {
				TraceSDUtil.debugUml2SDTraceEvents("Cancelled parseModel() after "+(System.currentTimeMillis()-start)+"ms"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				TraceSDUtil.debugUml2SDTraceEvents("End parseModel() after "+(System.currentTimeMillis()-start)+"ms"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}
	
	/**
	 * Parse the model again for a new translation on the same eObject (page changes, ...)
	 * @param update
	 */
	protected synchronized void updateSD(TraceInteractionUpdate update) {
		// Prepare for showing same date after update
		if (currentProcesses==null)
			return;
		Frame oldFrame = frame;
		if (frame != null &&
			!update.isDateSelectionChanged() &&
			!update.isPageChanged() &&
			!update.isFindRequired() &&
			!update.isFindInvocationRequired()) {
			double visibleDate = 0;
			currentProcesses.init();
			TraceSyncMessage firstSM = (TraceSyncMessage)(frame.getSyncMessage(frame.getFirstVisibleSyncMessage()));
			if (firstSM != null) {
				visibleDate = firstSM.getStartDate() + currentProcesses.getStartTime();
			} else {
				TraceSyncMessageReturn firstSMR = (TraceSyncMessageReturn)(frame.getSyncMessageReturn(frame.getFirstVisibleSyncMessageReturn()));
				if (firstSMR != null) {
					visibleDate = firstSMR.getStartDate() + currentProcesses.getStartTime();
				}
			}
			if (visibleDate != 0) {
				update.setDateSelectionChanged(true);
				update.setDateToShow(visibleDate);
			}
			if (TraceSDUtil.debugLoader) {
				TraceSDUtil.debugUml2SDTraceLoader("visibleDate="+visibleDate+" "+update.isPageChanged()+" "+update.isDateSelectionChanged()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		}
		createFrame();
		if (currentProcesses == null) {
			return;
		}
		try {
			if ((!update.isPageChanged()) &&
				(update.isLifelineLoaderChanged() ||
				 update.isInternalMessageFilteringChanged() ||
				 update.isMaxMessagesByPageChanged() ||
				 update.isFilteringChanged() ||
				 update.isHorizontalCollapsingChanged() ||
				 update.isVerticalCollapsingChanged())) {
				computeModel();
			}
			fillPage(update);
		} catch (FoundInPage foundInPage) {
			if (TraceSDUtil.debugLoader) {
				foundInPage.printStackTrace(System.out);
			}
			if (update.isDateSelectionChanged() ||
				update.isFindRequired() ||
				update.isFindInvocationRequired()) {
				update.setPageChanged(true);
				pageNumber = foundInPage.getPage();
				updateSD(update);
				return;
			}
		} catch (Throwable e) {
			e.printStackTrace();
		}
		if (!update.isPageChanged()) {
			if (update.isDateSelectionChanged() &&
				frame.lifeLinesCount() > 0 &&
				update.getVisibleGraphNode() == null) {
				// Not found because after the last event: show the last message of the last page
				update.setPageChanged(true);
				pageNumber = pages-1;
				createFrame();
				try {
					fillPage(update);
				} catch (FoundInPage foundInPage) {
					// Nothing to do
				}
				update.setPreviousVisibleGraphNode(null);
				update.setVisibleGraphNode(frame.getSyncMessageReturn(frame.syncMessageReturnCount()-1));
			} else if (update.isFindRequired() &&
					   (update.getFindResults() == null || update.getFindResults().size() == 0)) {
				// Not found!
				frame = oldFrame;
				return;
			} else if (update.isFindInvocationRequired() &&
					   update.getFindInvocationResult() == null) {
				// Not found!
				frame = oldFrame;
				return;
			}
		}
		if (view != null && !view.getSDWidget().isDisposed() && !isCurrentLoadCanceled()) {
			setFrameName();
			if (update.getVisibleGraphNode() != null) {
				GraphNode firstVisibleGraphNode = update.getPreviousVisibleGraphNode();
				if (firstVisibleGraphNode != null &&
					firstVisibleGraphNode instanceof BaseMessage &&
					update.getVisibleGraphNode() instanceof BaseMessage) {
				} else {
					firstVisibleGraphNode = update.getVisibleGraphNode();
				}
				if (update.getVisibleX() != 0) {
					view.setFrameAndEnsureVisibleSync(frame, Math.min(frame.getX()+frame.getWidth(),
							update.getVisibleX()), firstVisibleGraphNode.getY());
				} else {
					view.setFrameAndEnsureVisibleSync(frame, firstVisibleGraphNode);
				}
				if (update.isExternalDateSelectionChanged() && 
					firstVisibleGraphNode == update.getPreviousVisibleGraphNode()) {
					view.getTimeCompressionBar().highlightRegionSync(
							(BaseMessage)update.getPreviousVisibleGraphNode(),
						    (BaseMessage)update.getVisibleGraphNode());
				}
			} else {
				view.setFrameSync(frame);
			}
		}
	}
	
	/**
	 * @return
	 */
	protected void createFrame() {
		super.createFrame();
		frame.setLifelineCategories(traceCategories);
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("TraceInteractions.createFrame()"); //$NON-NLS-1$
		}
	}

	/**
	 * @param process
	 */
	public void setFrameName() {
		if (frame == null) {
			return;
		}
		if (!isEmpty()) {
			if (currentProcesses.size() == 1) {
				TRCProcess process = currentProcesses.get(0).getProcess();
				frame.setName(process.getAgent().getAgentProxy().getProcessProxy().getName()+
						  " [Pid "+process.getPid()+"]"+getDrillDownOrHideIndication()+pageMessage()); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				frame.setName(TraceUIMessages._150+getDrillDownOrHideIndication()+pageMessage());
			}
		} else {
			frame.setName(TraceUIMessages._151); //$NON-NLS-1$
		}
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("setFrameName() "+frame.getName()); //$NON-NLS-1$
		}
	}
	
	protected boolean isFullInfoCollected() {
		if (lifelineLoader instanceof ObjectInteractions)
		{
			if (currentProcesses != null && currentProcesses.size() > 0)
			{
				TRCProcess process;
				Iterator<TraceProcess> iter = currentProcesses.iterator();
				while (iter.hasNext())
				{
					 process = iter.next().getProcess();
					 if (process != null && process.getClassClass() != null && process.getObjects() != null
							 && process.getClassClass().getObjects() != null)
					 {
						 if (process.getClassClass().getObjects().size() != process.getObjects().size())
							 return true;
					 }

				}
			}
			return false;
		}
		return true;
	}
	
	protected boolean isEmpty()
	{
		if (currentProcesses == null || currentProcesses.size() < 1)
			return true;
		else
		{
			return false;
		}
	}

	/**
	 * This is the title
	 */
	public String getTitleString() {
		String ret = getFinalTitleString()+getDrillDownOrHideIndication();
		if (!isFullInfoCollected()) {
			ret += " "+TraceUIMessages._232; //$NON-NLS-1$
		}
		return ret + pageMessage();
	}
	
	private String getDrillDownOrHideIndication() {
		if (drillDownBreadcrump == null || drillDownBreadcrump.length() == 0) {
			if (currentFilters != null &&
				currentFilters.size() > 0 &&
				!hasPositiveFilters) {
				return " " + TraceUIMessages._231; //$NON-NLS-1$
			}
			return ""; //$NON-NLS-1$
		}
		return " " + TraceUIMessages._228 + drillDownBreadcrump; //$NON-NLS-1$
	}

	/**
	 * @return
	 */
	protected String pageMessage() {
		if (pages <= 1) {
			return ""; //$NON-NLS-1$
		}
		StringBuffer m = new StringBuffer(""); //$NON-NLS-1$
		m.append(" [").append((1+pageNumber)).append("/").append(pages).append(" ") //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			.append(TraceUIMessages._154).append("]"); //$NON-NLS-1$
		return m.toString();
	}
	
	private int totalNbMessages;
	private int pages;
	private ArrayList<TracePage> tracePages;
	
	/**
	 * Compute the number of pages with and also without internal messages
	 * @param invocations
	 */
	protected void computeModel() {
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("computeModel() "+maximumMessagesByPage); //$NON-NLS-1$
		}
		totalNbMessages = 0;
		instancesMap = new HashMap<Object, TraceLifelineDraft>();
		currentProcesses.init();
		for (TRCFullMethodInvocation methodInvocation = currentProcesses.consumeMethodInvocation();
		 	 methodInvocation != null &&
			 !isCurrentLoadCanceled();
		 	 methodInvocation = currentProcesses.consumeMethodInvocation()) {
			TRCFullMethodInvocation invokedBy = (TRCFullMethodInvocation)methodInvocation.getInvokedBy();
			if (isMessageCollapsed(invokedBy)) {
				continue;
			}
			EObject instanceFrom = null;
			EObject instanceTo = null;
			if (invokedBy != null) {
				instanceFrom = getLifelineEObjectFromMethodInvocation(invokedBy);
				instanceTo = getLifelineEObjectFromMethodInvocation(methodInvocation);
				if (internalMessagesFiltered &&	instanceFrom == instanceTo) {
					continue;
				}
			}
			TraceLifelineDraft traceLifelineTo = getTraceLifelineDraftForMethodInvocation(null, methodInvocation);
			TraceLifelineDraft traceLifelineFrom = null;
			if (invokedBy != null) {
				traceLifelineFrom = getTraceLifelineDraftForMethodInvocation(null, invokedBy);
			}
			if (traceLifelineTo.isFiltered() ||
				(traceLifelineFrom != null &&
				 traceLifelineFrom.isFiltered())) {
				continue;
			}
			String name = methodInvocation.getMethod().getName();
			if (isSyncMessageFiltered(name)) {
				continue;
			}
			// Count one way and return if set
			totalNbMessages++;
			if (methodInvocation.getExitTime() > 0.0) {
				totalNbMessages++;
			}
		}
		pages = totalNbMessages / maximumMessagesByPage;
		if (totalNbMessages % maximumMessagesByPage > 0) {
			pages++;
		}
		instancesMap = null;
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("computeModel() interrupted("+isCurrentLoadCanceled()+") has counted "+pages+" pages"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	}
	
	private HashMap<Object, TraceLifelineDraft> instancesMap;
	
	/**
	 * @param update
	 */
	public void fillPage(TraceInteractionUpdate update) {
		
		tracePages = new ArrayList<TracePage>();
		instancesMap = new HashMap<Object, TraceLifelineDraft>();
		traceThreadMap = new HashMap<TRCThread, TraceCallStack>();
		traceThreadList = new ArrayList<TraceCallStack>();
		lifelinesByName = new HashMap<String, List<TraceLifeline>>();
		nbMessages = 0;
		nbMessagesInThisPage = 0;
		hasNextPage = false;
		currentProcesses.init(); // init again
		for (TRCFullMethodInvocation methodInvocation = currentProcesses.consumeMethodInvocation();
	 	 	 methodInvocation != null &&
			 !isCurrentLoadCanceled() &&
			 nbMessagesInThisPage < maximumMessagesByPage;
	 	 	 methodInvocation = currentProcesses.consumeMethodInvocation()) {
			if (TraceSDUtil.debugLoader) {
				TraceSDUtil.debugUml2SDTraceLoader(nbMessages+":"+nbMessages/maximumMessagesByPage+" "+methodInvocation);			 //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			// Let's continue with pending message returns
			finishPendingReturns(update,
								 TIUtils.getAbsoluteEntryTime(methodInvocation),
								 methodInvocation.getTicket(),
								 methodInvocation.getThread(),
								 methodInvocation.getStackDepth());
			
			TraceSyncMessage syncMessage = createSyncMessage(methodInvocation, update);
			
			TraceCallStack callStack = getTraceCallStack(methodInvocation.getThread());
			callStack.pushCall(syncMessage, methodInvocation);
		}
		// Let's continue with pending message returns
		finishPendingReturns(update, Double.MAX_VALUE, Long.MAX_VALUE, null, Short.MAX_VALUE);

		// Let's do last execution occurrences
		Set<Entry<Object, TraceLifelineDraft>> c = instancesMap.entrySet();
		for (Iterator<Entry<Object, TraceLifelineDraft>> i = c.iterator(); i.hasNext(); ) {
			Entry<Object, TraceLifelineDraft> e = i.next();
			TraceLifelineDraft draft = e.getValue();
			if (TraceSDUtil.debugLoader) {
				TraceSDUtil.debugUml2SDTraceLoader(draft.getLifeline()+":"+draft.getNbUser()); //$NON-NLS-1$
			}
			if (draft.getNbUser() > 0) {
				draft.setEnd(nbMessagesInThisPage+1);
				if (draft.getStart() == Integer.MAX_VALUE) {
					draft.setStart(0);
				}
				setExecutionOccurence(draft);
			}
		}
		reorderLifelinesAgain();
		distinguishLifelinesByName();
		instancesMap = null;
		traceThreadMap = null;
		traceThreadList = null;
		lifelinesByName = null;
	}
	
	/**
	 * @param update
	 * @param uptodate
	 */
	protected void finishPendingReturns(TraceInteractionUpdate update,
									    double uptodate,
										long ticket,
										TRCThread thread,
										short stackDepth) {
		double traceThreadListFirstTime = getTraceThreadListFirstTime();
		long traceThreadListFirstTicket = getTraceThreadListFirstTicket();
		while (traceThreadListFirstTime < uptodate ||
			   (traceThreadListFirstTime == uptodate &&
			   	(traceThreadListFirstTicket < ticket ||
			   	 (traceThreadListFirstTicket == ticket &&
			   	  thread != null &&
			   	  getTraceThreadListFirstThread() == thread &&
			   	  stackDepth != Short.MAX_VALUE &&
			   	  getTraceThreadListFirstStackDepth() >= stackDepth)))) {
			TraceCallStack callStack = traceThreadList.get(0);
			TraceMessageAndCallAssociation tmaca = callStack.popCall();
			TRCFullMethodInvocation methodInvocation = tmaca.getMethodInvocation();
			if (methodInvocation.getExitTime() > 0.0) {
				TraceSyncMessageReturn syncMessageReturn = createSyncMessageReturn(methodInvocation, update);
				if (tmaca.getMessage() != null && syncMessageReturn != null) {
					syncMessageReturn.setMessage(tmaca.getMessage());
				}
			}
			traceThreadListFirstTime = getTraceThreadListFirstTime();
			traceThreadListFirstTicket = getTraceThreadListFirstTicket();
		}
	}

	/**
	 * @param methodInvocation
	 * @param update
	 */
	private TraceSyncMessage
			     createSyncMessage(TRCFullMethodInvocation methodInvocation,
								   TraceInteractionUpdate update) {
		TRCFullMethodInvocation invokedBy = (TRCFullMethodInvocation)methodInvocation.getInvokedBy();
		if (isMessageCollapsed(invokedBy)) {
			return null;
		}
		TraceLifelineDraft traceLifelineTo = getTraceLifelineDraftForMethodInvocation(frame, methodInvocation);
		TraceLifelineDraft traceLifelineFrom = null;
		if (invokedBy != null) {
			traceLifelineFrom = getTraceLifelineDraftForMethodInvocation(frame, invokedBy);
		}
		if (traceLifelineTo.isFiltered() ||
			(traceLifelineFrom != null &&
			 traceLifelineFrom.isFiltered())) {
			return null;
		}
		if (internalMessagesFiltered && traceLifelineFrom == traceLifelineTo) {
			return null;
		}
		if (isSyncMessageFiltered(methodInvocation.getMethod().getName())) {
			return null;
		}
		nbMessages++;
		String name = getMessageName(methodInvocation, traceLifelineFrom == null);
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("in:"+nbMessages+":"+nbMessages/maximumMessagesByPage+" "+name+" "+TraceInteractionUtils.getAbsoluteEntryTime(methodInvocation)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
		if (nbMessagesInThisPage == maximumMessagesByPage) {
			hasNextPage = true;
			return null;
		}
		if (traceLifelineFrom != null && traceLifelineFrom != traceLifelineTo) {
			traceLifelineFrom.addAUser();
		}
		traceLifelineTo.addAUser();
		if ((nbMessages-1) % maximumMessagesByPage == 0) {
			tracePages.add(new TracePage(TIUtils.getAbsoluteEntryTime(methodInvocation)));
		} else if ((nbMessages-1) % maximumMessagesByPage == maximumMessagesByPage-1) {
			tracePages.get(tracePages.size()-1).setLastTime(TIUtils.getAbsoluteExitTime(methodInvocation));
		}
		if (update.isPageChanged() &&
			(nbMessages-1) < pageNumber*maximumMessagesByPage) {
			return null;
		}
		if (!update.isPageChanged()) {
			if (update.isDateSelectionChanged()) {
				if (TIUtils.getAbsoluteEntryTime(methodInvocation) < update.getDateToShow()) {
					return null;
				} else {
					throw new FoundInPage(tracePages.size()-1);
				}
			}
			if (update.isFindRequired() &&
				update.getFindCriteria().isSyncMessageSelected()) {
				if (update.getFindStartingPage() <= tracePages.size()-1 &&
					TIUtils.matchCriteria(name, update.getFindCriteria())) {
					throw new FoundInPage(tracePages.size()-1);
				} else {
					return null;
				}
			}
			if (update.isFindInvocationRequired()) {
				if (!update.isFindInvocationReturn() &&
				    methodInvocation == update.getFindInvocationModel()) {
					throw new FoundInPage(tracePages.size()-1);
				} else {
					return null;
				}
			}
		}
		if (traceLifelineFrom != null && traceLifelineFrom != traceLifelineTo && traceLifelineFrom.getNbUser() == 1) {
			traceLifelineFrom.setStart(nbMessagesInThisPage+1);
		}
		if (traceLifelineTo.getNbUser() == 1) {
			traceLifelineTo.setStart(nbMessagesInThisPage+1);
		}
		TraceSyncMessage syncMessage;
		if (isMessageCollapsed(methodInvocation)) {
			syncMessage = new TraceCollapsedSyncMessage();
		} else if (methodInvocation.getInvokes() != null &&
				   methodInvocation.getInvokes().size() > 0) {
			syncMessage = new TraceCollapsableSyncMessage();
		} else {
			syncMessage = new TraceSyncMessage();
		}
		if (update.isPageChanged()) {
			if (update.isDateSelectionChanged()) {
				if (TIUtils.getAbsoluteEntryTime(methodInvocation) < update.getDateToShow()) {
					update.setPreviousVisibleGraphNode(syncMessage);
				} else if (update.getVisibleGraphNode() == null) {
					update.setVisibleGraphNode(syncMessage);
				}
			}
			if (update.isFindRequired() &&
				update.getFindCriteria().isSyncMessageSelected()) {
				if (TIUtils.matchCriteria(name, update.getFindCriteria())) {
					update.addFindResult(syncMessage);
				}
			}
			if (update.isFindInvocationRequired() &&
				!update.isFindInvocationReturn() &&
				methodInvocation == update.getFindInvocationModel()) {
					update.setFindInvocationResult(syncMessage);
			}
		}
		syncMessage.model = methodInvocation;
		if (traceLifelineFrom != null) {
			traceLifelineFrom.getLifeline().setCurrentEventOccurrence(nbMessagesInThisPage+1);
			syncMessage.setStartLifeline(traceLifelineFrom.getLifeline());
		} else { 
			traceLifelineTo.getLifeline().setCurrentEventOccurrence(nbMessagesInThisPage+1);
		}
		syncMessage.setName(name);
		syncMessage.setEndLifeline(traceLifelineTo.getLifeline());
		syncMessage.setTime(TIUtils.getAbsoluteEntryTime(methodInvocation));
		if (isCurrentLoadCanceled()) {
			return null;
		}
		frame.addMessage(syncMessage);
		nbMessagesInThisPage++;
		return syncMessage;
	}
	
	/**
	 * @param methodInvocation
	 * @param update
	 */
	private TraceSyncMessageReturn
				 createSyncMessageReturn(TRCFullMethodInvocation methodInvocation,
										 TraceInteractionUpdate update) {
		TRCFullMethodInvocation invokedBy = (TRCFullMethodInvocation)methodInvocation.getInvokedBy();
		if (isMessageCollapsed(invokedBy)) {
			return null;
		}
		TraceLifelineDraft traceLifelineTo = getTraceLifelineDraftForMethodInvocation(frame, methodInvocation);
		TraceLifelineDraft traceLifelineFrom = null;
		if (invokedBy != null) {
			traceLifelineFrom = getTraceLifelineDraftForMethodInvocation(frame, invokedBy);
		}
		if (traceLifelineTo.isFiltered() ||
			(traceLifelineFrom != null &&
			 traceLifelineFrom.isFiltered())) {
			return null;
		}
		if (internalMessagesFiltered && traceLifelineFrom == traceLifelineTo) {
			return null;
		}
		if (isSyncMessageFiltered(methodInvocation.getMethod().getName())) {
			return null;
		}
		nbMessages++;
		String name = getMessageName(methodInvocation, traceLifelineFrom == null);
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("out:"+nbMessages+":"+nbMessages/maximumMessagesByPage+" "+name+" "+TraceInteractionUtils.getAbsoluteExitTime(methodInvocation)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
		if (nbMessagesInThisPage == maximumMessagesByPage) {
			hasNextPage = true;
			return null;
		}
		if (traceLifelineFrom != null && traceLifelineFrom != traceLifelineTo) {
			traceLifelineFrom.removeAUser();
		}
		traceLifelineTo.removeAUser();
		if ((nbMessages-1) % maximumMessagesByPage == 0) {
			tracePages.add(new TracePage(TIUtils.getAbsoluteExitTime(methodInvocation)));
		} else if ((nbMessages-1) % maximumMessagesByPage == maximumMessagesByPage-1) {
			tracePages.get(tracePages.size()-1).setLastTime(TIUtils.getAbsoluteExitTime(methodInvocation));
		}
		if (update.isPageChanged() &&
			(nbMessages-1) < pageNumber*maximumMessagesByPage) {
			return null;
		}
		if (!update.isPageChanged()) {
			if (update.isDateSelectionChanged()) {
				if (TIUtils.getAbsoluteExitTime(methodInvocation) < update.getDateToShow()) {
					return null;
				} else {
					throw new FoundInPage(tracePages.size()-1);
				}
			}
			if (update.isFindRequired() &&
				update.getFindCriteria().isSyncMessageSelected()) {
				if (update.getFindStartingPage() <= tracePages.size()-1 &&
					TIUtils.matchCriteria(name, update.getFindCriteria())) {
					throw new FoundInPage(tracePages.size()-1);
				} else {
					return null;
				}
			}
			if (update.isFindInvocationRequired()) {
				if (update.isFindInvocationReturn() &&
				    methodInvocation == update.getFindInvocationModel()) {
					throw new FoundInPage(tracePages.size()-1);
				} else {
					return null;
				}
			}
		}
		if (traceLifelineFrom != null && traceLifelineFrom != traceLifelineTo && traceLifelineFrom.getNbUser() == 0) {
			traceLifelineFrom.setEnd(nbMessagesInThisPage+1);
			setExecutionOccurence(traceLifelineFrom);
		}
		if (traceLifelineTo.getNbUser() == 0) {
			traceLifelineTo.setEnd(nbMessagesInThisPage+1);
			setExecutionOccurence(traceLifelineTo);
		}
		TraceSyncMessageReturn syncMessageReturn;
		if (isMessageCollapsed(methodInvocation)) {
			syncMessageReturn = new TraceCollapsedSyncMessageReturn();
		} else if (methodInvocation.getInvokes() != null &&
				   methodInvocation.getInvokes().size() > 0) {
			syncMessageReturn = new TraceCollapsableSyncMessageReturn();
		} else {
			syncMessageReturn = new TraceSyncMessageReturn();
		}
		if (update.isPageChanged()) {
			if (update.isDateSelectionChanged()) {
				if (TIUtils.getAbsoluteExitTime(methodInvocation) < update.getDateToShow()) {
					update.setPreviousVisibleGraphNode(syncMessageReturn);
				} else if (update.getVisibleGraphNode() == null) {
					update.setVisibleGraphNode(syncMessageReturn);
				}
			}
			if (update.isFindRequired() &&
				update.getFindCriteria().isSyncMessageSelected()) {
				if (TIUtils.matchCriteria(name, update.getFindCriteria())) {
					update.addFindResult(syncMessageReturn);
				}
			}
			if (update.isFindInvocationRequired() &&
				update.isFindInvocationReturn() &&
				methodInvocation == update.getFindInvocationModel()) {
				update.setFindInvocationResult(syncMessageReturn);
			}
		}
		syncMessageReturn.model = methodInvocation;
		if (traceLifelineFrom != null) {
			traceLifelineFrom.getLifeline().setCurrentEventOccurrence(nbMessagesInThisPage+1);
			syncMessageReturn.setEndLifeline(traceLifelineFrom.getLifeline());
		} else { 
			traceLifelineTo.getLifeline().setCurrentEventOccurrence(nbMessagesInThisPage+1);
		}
		syncMessageReturn.setName(name);
		syncMessageReturn.setStartLifeline(traceLifelineTo.getLifeline());
		syncMessageReturn.setTime(TIUtils.getAbsoluteExitTime(methodInvocation));
		if (isCurrentLoadCanceled()) {
			return null;
		}
		frame.addMessage(syncMessageReturn);
		nbMessagesInThisPage++;
		return syncMessageReturn;
	}
	
	/**
	 * @param methodInvocation
	 * @param withThread
	 * @return
	 */
	private String getMessageName(TRCFullMethodInvocation methodInvocation, boolean withThread) {
		if (withThread) {
			
			StringBuffer buffer = new StringBuffer(methodInvocation.getMethod().getName());
			return buffer.append(" (").append(methodInvocation.getThread().getName()).append(")").toString(); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			return methodInvocation.getMethod().getName();
		}
	}
	
	/**
	 * Get the instance corresponding to this method invocation or create it
	 * @param f
	 * @param mi
	 * @return
	 */
	protected TraceLifelineDraft 
				getTraceLifelineDraftForMethodInvocation
					(Frame f, TRCFullMethodInvocation mi) {
		EObject obj = getLifelineEObjectFromMethodInvocation(mi);
		String name = getLifeLineTitle(obj, true);
		String id = getLifeLineId(obj);
		TraceLifelineDraft draft = instancesMap.get(id);
		if (draft == null) {
			draft = new TraceLifelineDraft(obj);
			if (isLifelineFiltered(obj) || isLifelineLocallyFiltered(name, mi)) {
				draft.setFiltered(true);
				instancesMap.put(id, draft);
				return draft;
			}
			List<Object> list = collapsedLifelines != null ?
							 collapsedLifelines.get(id) : 
							 null;
			if (list != null) {
				for (List<Object> l = (ArrayList<Object>)collapsedLifelines.get(list);
					 l != null;
					 list = l, l = collapsedLifelines.get(l));
				name = getCollapsedLifelineName(list);
				if (name.length() > 300) {
					name = name.substring(0, 297)+"..."; //$NON-NLS-1$
				}
				if (isLifelineFiltered(obj) || isLifelineLocallyFiltered(name, mi)) {
					draft.setFiltered(true);
					putInstancesMap(list, draft);
					return draft;
				}
				if (f == null) {
					return draft;
				}
				TraceCollapsedLifelines traceCollapsedLifelines = new TraceCollapsedLifelines();
				draft.setLifeline(traceCollapsedLifelines);
				traceCollapsedLifelines.setCollapsedLifelines(list);
				traceCollapsedLifelines.setName(name);
				putInstancesMap(list, draft);
				traceCollapsedLifelines.setCategory(CATEGORY_COLLAPSED);
			} else {
				if (f == null) {
					return draft;
				}
				TraceLifeline traceLifeline = new TraceLifeline();
				draft.setLifeline(traceLifeline);
				traceLifeline.setEObject(obj);
				instancesMap.put(id, draft);
				traceLifeline.setName(name);
				addLifelineByName(traceLifeline);
				traceLifeline.setCategory(getLifeLineCategory(obj));
			}
			f.addLifeLine(draft.getLifeline());
			draft.setStart(Integer.MAX_VALUE);
			draft.setEnd(0);
		}
		return draft;
	}
	
	/**
	 * Avoiding two lifelines with the same name -> needs a second stage to distinguish their names
	 */
	private HashMap<String, List<TraceLifeline>> lifelinesByName;
	private ArrayList<List<TraceLifeline>> lifelinesToDistinguish;
	
	private void addLifelineByName(TraceLifeline traceLifeline) {
		List<TraceLifeline> list = lifelinesByName.get(traceLifeline.getName());
		if (list == null) {
			list = new ArrayList<TraceLifeline>(1);
			lifelinesByName.put(traceLifeline.getName(), list);
		}
		list.add(traceLifeline);
		if (list.size() == 2) {
			if (lifelinesToDistinguish == null) {
				lifelinesToDistinguish = new ArrayList<List<TraceLifeline>>(1);
			}
			lifelinesToDistinguish.add(list);
		}
	}
	
	private void distinguishLifelinesByName() {
		if (lifelinesToDistinguish == null) {
			// Nothing to distinguish
			return;
		}
		for (Iterator<List<TraceLifeline>> i = lifelinesToDistinguish.iterator(); i.hasNext(); ) {
			List<TraceLifeline> list = i.next();
			for (int j = 0; j < list.size(); j++) {
				TraceLifeline traceLifeline = list.get(j);
				traceLifeline.setName(getDistinguishedLifeLineTitle(traceLifeline.getEObject(), j));
			}
		}
		lifelinesToDistinguish = null;
	}

	/**
	 * @param draft
	 * @param list
	 */
	@SuppressWarnings("unchecked")
	private void putInstancesMap(List<Object> list, TraceLifelineDraft draft) {
		for (Iterator<Object> i = list.iterator(); i.hasNext(); ) {
			Object oo = i.next();
			if (oo instanceof String) {
				instancesMap.put(oo, draft);
			} else if (oo instanceof ArrayList) {
				putInstancesMap((List<Object>)oo, draft);
			} // else ignored
		}
	}

	/**
	 * @param draft
	 * @param list
	 */
	@SuppressWarnings("unchecked")
	private String getCollapsedLifelineName(List<Object> list) {
		StringBuffer ret = new StringBuffer("["); //$NON-NLS-1$
		int j = 0;
		for (Iterator<Object> i = list.iterator(); i.hasNext(); j++) {
			Object oo = i.next();
			if (oo instanceof String) {
				if (j > 0) {
					ret.append(","); //$NON-NLS-1$
				}
				String[] split = ((String)oo).split("#"); //$NON-NLS-1$
				ret.append(split.length == 2 ? split[1] : (String)oo);
			} else if (oo instanceof List) {
				if (j > 0) {
					ret.append(","); //$NON-NLS-1$
				}
				ret.append(getCollapsedLifelineName((List<Object>)oo));
			} // else ignored
		}
		return ret.append("]").toString(); //$NON-NLS-1$
	}

	
	/**
	 * @param part
	 * @param selection
	 */
	private void externalDateSelectionChanged(IDateSelection selection,
											  TraceInteractionUpdate update) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("External Date selection is "+selection); //$NON-NLS-1$
		}
		IDateSelection ids = (IDateSelection)selection;
		EObject eo = selection.getEObject();
		if (currentProcesses == null ||
			!(eo == null || (eo instanceof TRCProcess && currentProcesses.contains((TRCProcess)eo)))) {
			return;
		}
		double dateSelection, absoluteDateSelection;
		dateSelection = ids.getStartDate();
		if (eo != null) {
			TRCProcess process = (TRCProcess)eo;
			absoluteDateSelection = dateSelection;
			double absoluteEntryTime = TIUtils.getAbsoluteEntryTime(process.getAgent());
			if (absoluteDateSelection < absoluteEntryTime) {
				// determine if we received an absolute or a process relative date
				absoluteDateSelection += absoluteEntryTime;
			} else {
				dateSelection -= absoluteEntryTime;
			}
			if (TraceSDUtil.debugEvents) {
				TraceSDUtil.debugUml2SDTraceEvents("dateSelection="+dateSelection+"; absoluteDateSelection="+absoluteDateSelection+"; getAbsoluteEntryTime(process)="+TraceInteractionUtils.getAbsoluteEntryTime(process)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		    }
		} else {
			absoluteDateSelection = dateSelection;
		}
		
		// Is it in the current page? (this question is answered with date relative to the process)
		if (ids.getMeaning() == IDateSelection.NOW) {
			ITimeRange bounds[] = new ITimeRange[2];
			if (frame.findDateBounds(absoluteDateSelection, bounds)) {
				if (bounds[0] instanceof GraphNode) {
					view.getSDWidget().ensureVisible((GraphNode)bounds[0]);
					view.getTimeCompressionBar().highlightRegion((BaseMessage)bounds[0], (BaseMessage)bounds[1]);
					if (TraceSDUtil.debugEvents) {
						TraceSDUtil.debugUml2SDTraceEvents("Found a graph node"); //$NON-NLS-1$
					}
					return;
				}
				if (TraceSDUtil.debugEvents) {
					TraceSDUtil.debugUml2SDTraceEvents("Found something else than a graph node: ignored"); //$NON-NLS-1$
				}
				frame.resetTimeCompression();
				return;
			}
			// How many pages in there?
			frame.resetTimeCompression();
			if (pages == 1) {
				ITimeRange sm = bounds[0] != null ? bounds[0] : bounds[1];
				if (sm != null && sm instanceof GraphNode) {
					view.getSDWidget().ensureVisible((GraphNode)sm);
					if (TraceSDUtil.debugEvents) {
						TraceSDUtil.debugUml2SDTraceEvents(bounds[0] == null?"Before any graph node":"After any graph node"); //$NON-NLS-1$ //$NON-NLS-2$
					}
					return;
				}
				if (TraceSDUtil.debugEvents) {
					TraceSDUtil.debugUml2SDTraceEvents("Found something else than a graph node (#2): ignored"); //$NON-NLS-1$
				}
				return;
			}
			// Look in previous pages
			if (tracePages != null &&
				tracePages.size() > 0 &&
				dateSelection <= tracePages.get(tracePages.size()-1).getLastTime()) {
				for (int i = 0; i < tracePages.size(); i++) {
					TracePage tp = tracePages.get(i);
					if (dateSelection >= tp.getFirstTime() &&
						dateSelection <= tp.getLastTime()) {
						update.setDateSelectionChanged(true);
						update.setExternalDateSelectionChanged(true);
						update.setDateToShow(absoluteDateSelection);
						update.setPageChanged(true);
						pageNumber = i;
						if (TraceSDUtil.debugEvents) {
							TraceSDUtil.debugUml2SDTraceEvents("Found in a previous page"); //$NON-NLS-1$
						}
						return;
					}
				}
			}
			// Worst case: must look in all pages for an absolute date (no longer relative)
			update.setDateSelectionChanged(true);
			update.setExternalDateSelectionChanged(true);
			update.setDateToShow(absoluteDateSelection);
			if (TraceSDUtil.debugEvents) {
				TraceSDUtil.debugUml2SDTraceEvents("Looking in all pages"); //$NON-NLS-1$
			}
		}
	}
	
	/**
	 * Try to find, select and display a given method invocation message or message return.
	 * @param model method invocation to look for
	 * @param isReturn true if looking for the corresponding message return, false for the message
	 */
	public static void findInvocation(TRCMethodInvocation model, boolean isReturn) {
		if (instance != null && instance.frame != null) {
			instance.updateOnFindInvocation(model, isReturn);
		}
	}
	
	/**
	 * 
	 */
	private void updateOnFindInvocation(TRCMethodInvocation model, boolean isReturn) {
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setFindInvocationRequired(true);
		update.setFindInvocationModel(model);
		update.setFindInvocationReturn(isReturn);
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("updateOnFindInvocation: model="+model+" isReturn="+isReturn); //$NON-NLS-1$ //$NON-NLS-2$
		}
		updateSD(update);
		if (update.isFindInvocationRequired() &&
			update.getFindInvocationResult() != null) {
			final GraphNode graphNode = (GraphNode)update.getFindInvocationResult();
			Display.getDefault().syncExec(new Runnable() {
				public void run() {
					if (frame == null) {
						return;
					}
					view.getSDWidget().moveTo(graphNode);
				}
			});
		}
	}

	/**
	 * @param selection must contain a IDateSelection with an absolute time, the meaning NOW and
	 * no need for getEObject() to return a process (a null EObject would be fine).
	 */
	public static void setExternalDateSelection(IDateSelection selection) {
		if (instance != null && instance.frame != null) {
			instance.updateOnExternalDateSelection(selection);
		}
	}

	/**
	 * 
	 */
	private void updateOnExternalDateSelection(IDateSelection selection) {
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		externalDateSelectionChanged(selection, update);
		if (TraceSDUtil.debugLoader) {
			TraceSDUtil.debugUml2SDTraceLoader("updateOnExternalDateSelection: selection="+selection); //$NON-NLS-1$
		}
		if (update.needsUpdate()) {
			updateSDBackground(update);
		}
	}

	protected Timer subscribeSelectionTimer;

	/**
	 * @param selection
	 */
	private void subscribeForDateSelectionChanged(final IDateSelection selection) {
		if (subscribeSelectionTimer != null) {
			subscribeSelectionTimer.cancel();
		}
		subscribeSelectionTimer = new Timer();
		subscribeSelectionTimer.schedule(new TimerTask() {
			public void run() {
				Display.getDefault().syncExec(new Runnable() {
					public void run() {
						if (frame == null) {
							return;
						}
						updateOnExternalDateSelection(selection);
					}
				});
				subscribeSelectionTimer = null;
			}
		}, 50);
	}
		
	/**
	 * @see org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions#finishExternalExtendedSelection(org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceInteractionUpdate)
	 */
	protected void finishExternalExtendedSelection(TraceInteractionUpdate update) {
		// Nothing to do
	}

	/**
	 * @param part
	 * @param selection
	 * @param update
	 */
	protected boolean externalExtendedSelectionChanged(IWorkbenchPart part,
										    		   Object selection,
													   TraceInteractionUpdate update) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("TraceInteractions: External extended selection from "+part+" is "+selection); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (selection instanceof IDateSelection) {
			if (TraceSDUtil.debugEvents) {
				TraceSDUtil.debugUml2SDTraceEvents("TraceInteractions: IDateSelection: "+((IDateSelection)selection).getStartDate()); //$NON-NLS-1$
			}
    		subscribeForDateSelectionChanged((IDateSelection)selection);
    		return true;
    	} else if (selection instanceof IEObjectSelection) {
			EObject eObject = ((IEObjectSelection)selection).getEObject();
			if (TraceSDUtil.debugEvents) {
				TraceSDUtil.debugUml2SDTraceEvents("TraceInteractions: IEObjectSelection: "+eObject); //$NON-NLS-1$
			}
    		return false;
    	}
		return false;
	}


	/**
	 * implementation of ISDPagingProvider
	 */
	public boolean hasNextPage() {
		return hasNextPage;
	}
	public boolean hasPrevPage() {
		return pageNumber > 0;
	}
	public void nextPage() {
		pageNumber++;
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setPageChanged(true);
		updateSDBackground(update);
	}
	public void prevPage() {
		pageNumber--;
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setPageChanged(true);
		updateSDBackground(update);
	}
	
	/**
	 * Implementation of ISDAdvancedPagingProvider
	 */
	public int maxItemsByPageCount() {
		return maximumMessagesByPage;
	}
	public int itemsTotalCount() {
		return totalNbMessages;
	}
	public String noItemsText() {
		return TraceUIMessages._155; //$NON-NLS-1$
	}
	public String oneItemText() {
		return TraceUIMessages._156; //$NON-NLS-1$
	}
	public String itemsText() {
		return TraceUIMessages._157; //$NON-NLS-1$
	}
	public int currentPage() {
		return pageNumber;
	}
	public int pagesCount() {
		return pages;
	}
	public void pageSettingsChanged(int maxItemsByPage_, int pageNumber_) {
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setMaxMessagesByPageChanged(maximumMessagesByPage != maxItemsByPage_);
		maximumMessagesByPage = maxItemsByPage_;
		update.setPageChanged(pageNumber != pageNumber_);
		pageNumber = pageNumber_;
		if (update.isMaxMessagesByPageChanged() ||
			update.isPageChanged()) {
			updateSDBackground(update);
		}
	}
	public void pageNumberChanged(int pageNumber_) {
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setPageChanged(pageNumber != pageNumber_);
		pageNumber = pageNumber_;
		if (update.isPageChanged()) {
			updateSDBackground(update);
		}
	}

	/**
	 * @see org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions#notFoundYet(org.eclipse.hyades.uml2sd.ui.actions.widgets.Criteria)
	 */
	public boolean notFoundYet(Criteria toSearch) {
		// looking in other pages if any
		if (pages > 1 && pageNumber+1 < pages) {
			TraceInteractionUpdate update = new TraceInteractionUpdate();
			update.setFindRequired(true);
			update.setFindCriteria(toSearch);
			update.setFindStartingPage(pageNumber+1);
			updateSD(update);
			ArrayList<GraphNode> findResults_ = update.getFindResults();
			if (findResults_ != null && !findResults_.isEmpty()) {
				findResults = findResults_;
				currentFindIndex = 0;
				GraphNode current = findResults.get(currentFindIndex);
				view.getSDWidget().moveTo(current);
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Implementation of ISDInternalMesFilterProvider
	 */
	private static final String INTERNAL_MESSAGES = "internalMessages"; //$NON-NLS-1$
	
	public void setInternalMessageFiltered(boolean value) {
		if (internalMessagesFiltered != value) {
			internalMessagesFiltered = value;
			DialogSettings settings = (DialogSettings)TraceUIPlugin.getDefault().getDialogSettings();
			settings.put(INTERNAL_MESSAGES, internalMessagesFiltered);
			TraceInteractionUpdate update = new TraceInteractionUpdate();
			update.setInternalMessageFilteringChanged(true);
			updateSDBackground(update);
		}
	}
	
	public boolean getInternalMessageFiltered() {
		DialogSettings settings = (DialogSettings)TraceUIPlugin.getDefault().getDialogSettings();
		internalMessagesFiltered = settings.getBoolean(INTERNAL_MESSAGES); // defaults to false
		return internalMessagesFiltered;
	}
	
	private List<FilterCriteria> currentFilters;
	private boolean hasPositiveFilters = false;
	private String drillDownBreadcrump = ""; //$NON-NLS-1$
	private UndoDrillDownMenuAction undoDrillDownMenuAction;
	
	private static FilterCriteria unmatchFilterCriteria = new FilterCriteria(null, true, false, null);

	private FilterCriteria isLifelineFiltered(String name, String loaderClassName) {
		for (Iterator<FilterCriteria> i = currentFilters.iterator(); i.hasNext(); ) {
			FilterCriteria tf = i.next();
			if (tf.isActive() && tf.getCriteria().isLifeLineSelected()) {
				if (tf.getLoaderClassName() != null &&
					tf.isPositive() &&
					tf.getLoaderClassName().equals(loaderClassName)) {
					if (TIUtils.matchCriteria(name, tf.getCriteria())) {
						if (TraceSDUtil.debugMatch) {
							TraceSDUtil.debugUml2SDTraceMatch("isLifelineFiltered("+name+" ,"+loaderClassName+") returns "+tf); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						}
						return tf;
					}
					if (TraceSDUtil.debugMatch) {
						TraceSDUtil.debugUml2SDTraceMatch("isLifelineFiltered("+name+" ,"+loaderClassName+") returns "+unmatchFilterCriteria); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
					return unmatchFilterCriteria;
			    } else if (tf.getLoaderClassName() == null &&
			    		   !tf.isPositive()) {
			    	if (TIUtils.matchCriteria(name, tf.getCriteria())) {
			    		if (TraceSDUtil.debugMatch) {
			    			TraceSDUtil.debugUml2SDTraceMatch("isLifelineFiltered("+name+" ,"+loaderClassName+") returns "+unmatchFilterCriteria); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			    		}
						return unmatchFilterCriteria;
					}
			    }
			}
		}
		if (TraceSDUtil.debugMatch) {
			TraceSDUtil.debugUml2SDTraceMatch("isLifelineFiltered("+name+" ,"+loaderClassName+") returns null"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		return null;
	}

	private boolean isLifelineLocallyFiltered(String name, TRCFullMethodInvocation mi) {
		if (currentFilters != null && currentFilters.size() > 0) {
			FilterCriteria tf = isLifelineFiltered(name, lifelineLoader.getClass().getName());
			if (tf != null) {
				return !tf.isPositive();
			}
			if (hasPositiveFilters) {
				boolean hasPositiveFilterMatching = false;
				for (ILifelineLoader loader = lifelineLoader.getParent();
					 loader != null;
				 	 loader = loader.getParent()) {
					tf = isLifelineFiltered(loader.getLifeLineTitle(
												loader.getLifelineEObjectFromMethodInvocation(mi), true),
											loader.getClass().getName());
					if (tf != null && tf.isPositive()) {
						hasPositiveFilterMatching = true;
					} else if (tf == unmatchFilterCriteria) {
						return true;
					}
				}
				return !hasPositiveFilterMatching;
			}
		}
		return false;
	}
	
	private boolean isSyncMessageFiltered(String name) {
		if (currentFilters != null) {
			for (Iterator<FilterCriteria> i = currentFilters.iterator(); i.hasNext(); ) {
				FilterCriteria tf = i.next();
				if (tf.isActive() && tf.getCriteria().isSyncMessageSelected()) {
					if (TIUtils.matchCriteria(name, tf.getCriteria())) {
						return !tf.isPositive();
					}
				}
			}
		}
		return false;
	}
	
	private void filterGraphNodeInOrOut(List<GraphNode> graphNodes, boolean in, String loaderClassName) {
		if (currentFilters == null) {
			currentFilters = new ArrayList<FilterCriteria>();
		}
		for (Iterator<GraphNode> i = graphNodes.iterator(); i.hasNext(); ) {
			GraphNode gn = i.next();
			Criteria c = new Criteria();
			if (gn instanceof IEObjectSelection) {
				EObject eo = (EObject)((IEObjectSelection)gn).getEObject();
				if (eo instanceof TRCFullMethodInvocation) {
					c.setExpression(((TRCFullMethodInvocation)eo).getMethod().getName());
				}
			}
			c.setCaseSenstiveSelected(true);
			if (gn instanceof TraceLifeline) {
				c.setExpression(lifelineLoader.getLifeLineTitle(((TraceLifeline)gn).getEObject(), true));
				c.setLifeLineSelected(true);
			} else if (gn instanceof TraceSyncMessage) {
				c.setExpression(((TRCFullMethodInvocation)((TraceSyncMessage)gn).getEObject()).getMethod().getName());
				c.setSyncMessageSelected(true);
			} else if (gn instanceof TraceSyncMessageReturn) {
				c.setExpression(((TRCFullMethodInvocation)((TraceSyncMessageReturn)gn).getEObject()).getMethod().getName());
				c.setSyncMessageSelected(true);
			}
			FilterCriteria fc;
			int position;
			if (in) {
				position = 0;
				fc = new FilterCriteria(c, true, true, loaderClassName);
			} else {
				position = -1;
				fc = new FilterCriteria(c, true, false);
			}
			FilterCriteria efc = FilterCriteria.find(fc, currentFilters);
			if (efc == null) {
				if (position == 0) {
					currentFilters.add(0, fc);
				} else {
					currentFilters.add(fc);
				}
			} else {
				efc.setActive(true);
			}
		}
		updateHasPositiveLifelineFilters();
		FilterListDialog.saveFiltersCriteria(currentFilters);
	}

	/**
	 * 
	 */
	public void filterSelectedGraphNodes(ArrayList<GraphNode> graphNodes) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("filterGraphNodes("+(graphNodes!=null ? graphNodes.size() : 0)+" selected graph nodes)"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		filterGraphNodeInOrOut(graphNodes, false, null);
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setFilteringChanged(true);
		updateSDBackground(update);
	}
	
	/**
	 * Come back from the filters dialog
	 */
	public boolean filter(List<FilterCriteria> filters) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("filter("+(filters!=null ? filters.size() : 0)+" filters)"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		currentFilters = filters;
		updateHasPositiveLifelineFilters();
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setFilteringChanged(true);
		updateSDBackground(update);
		return true;
	}

	/**
	 * @see org.eclipse.hyades.uml2sd.ui.actions.provider.IExtendedFilterProvider#getFilterAction()
	 */
	public Action getFilterAction() {
		return (Action) FilterActions.getFiltersSelectionAction(this);
	}

	/**
	 * @see org.eclipse.hyades.trace.ui.filters.IFilterViewer#filterUpdate(org.eclipse.hyades.models.hierarchy.extensions.SimpleSearchQuery)
	 */
	public void filterUpdated() {
		filter = filterQueryProvider.getCurrentFilter();
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setFilteringChanged(true);
		updateSDBackground(update);
	}

	/**
	 * @see org.eclipse.hyades.trace.ui.filters.IFilterViewer#setFilterQueryProvider(org.eclipse.hyades.trace.ui.filters.IFilterQueryProvider)
	 */
	public void setFilterQueryProvider(IFilterQueryProvider filterQueryProvider_) {
		filterQueryProvider = filterQueryProvider_;
		SimpleSearchQuery currentFilter = filterQueryProvider.getCurrentFilter();
		if (currentFilter != null ) {
			filter = EcoreUtil.copy(currentFilter);
		} else {
			filter = null;
		}
	}

	/**
	 * @see org.eclipse.hyades.trace.ui.filters.IFilterViewer#getFilterScope()
	 */
	public String[] getFilterScope() {
		String[] scope = new String[1];
		scope[0] = "ProfilingAgentScope"; //$NON-NLS-1$
		return scope;
	}

	private boolean isLifelineFiltered(EObject source) {
		if (filter == null) {
			return false;
		}
		return !FilterQuery.isMatchingFilter(source, filter);
	}

	/**
	 * @param filters
	 */
	private void updateHasPositiveLifelineFilters() {
		drillDownBreadcrump = ""; //$NON-NLS-1$
		hasPositiveFilters = false;
		boolean drillDownActive = false;
		if (currentFilters != null) {
			for (Iterator<FilterCriteria> i = currentFilters.iterator(); i.hasNext(); ) {
				FilterCriteria fc = i.next();
				if (fc.isActive() && fc.isPositive() && fc.getCriteria().isLifeLineSelected()) {
					hasPositiveFilters = true;
					if (fc.getLoaderClassName() != null) {
						drillDownBreadcrump = " > "+ //$NON-NLS-1$
							fc.getCriteria().getGraphNodeSummary(this, fc.getLoaderClassName())+" "+ //$NON-NLS-1$
							fc.getCriteria().getExpression()+drillDownBreadcrump;
						drillDownActive = true;
					}
				}
			}
		}
		if (undoDrillDownMenuAction != null) {
			undoDrillDownMenuAction.setEnabled(hasPositiveFilters);
		}
		try {
			IContributionItem item = view.getViewSite().getActionBars().getToolBarManager().find("org.eclipse.hyades.uml2sd.trace.actions.internal.ResetDrillDown"); //$NON-NLS-1$
			if (item != null) {
				item.setVisible(drillDownActive);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	public void drillDown(List<GraphNode> graphNodes) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("drillDown("+(graphNodes!=null ? graphNodes.size() : 0)+" selected graph nodes)"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		filterGraphNodeInOrOut(graphNodes, true, lifelineLoader.getClass().getName());
		List<IConfigurationElement> loaders = SDUtil.getLoaders(view.getViewSite().getId());
		String className = null;
		for (Iterator<IConfigurationElement> i = loaders.iterator(); i.hasNext(); ) {
			IConfigurationElement c = i.next();
			if (c.getAttribute("class").equals(lifelineLoader.getClass().getName()) && //$NON-NLS-1$
				i.hasNext()) {
				c = i.next();
				className = c.getAttribute("class"); //$NON-NLS-1$
				break;
			}
		}
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setFilteringChanged(true);
		if (className != null) {
			setLifelineLoader((BaseLifelineLoader)LoadersManager.getLoadersManager().createLoaderInstance(className, getClass().getClassLoader()));
			SDUtil.getInstance().setLastLoader(lifelineLoader.getClass().getName(), view.getSite().getId());
			update.setLifelineLoaderChanged(true);
		}
		updateSDBackground(update);
	}
	
	private void undoDrillDown(boolean all) {
		if (currentFilters != null) {
			FilterCriteria lastRemovedFilterCriteria = null;
			ILifelineLoader parent = lifelineLoader;
			Object array[] = currentFilters.toArray();
			for (int i = 0; i < array.length; i++) {
				FilterCriteria fc = (FilterCriteria)array[i];
				if (fc.isActive() && fc.isPositive() && fc.getCriteria().isLifeLineSelected() &&
					fc.getLoaderClassName() != null) {
					currentFilters.remove(lastRemovedFilterCriteria = fc);
					if (parent != null) {
						parent = parent.getParent();
					}
					if (!all) {
						break;
					}
				}
			}
			updateHasPositiveLifelineFilters();
			FilterListDialog.saveFiltersCriteria(currentFilters);
			if (lastRemovedFilterCriteria != null &&
				parent != null &&
				parent.getClass().getName().equals(lastRemovedFilterCriteria.getLoaderClassName())) {
				setLifelineLoader(lastRemovedFilterCriteria.getLoaderClassName());
			} else {
				TraceInteractionUpdate update = new TraceInteractionUpdate();
				update.setFilteringChanged(true);
				updateSDBackground(update);
			}
		}
	}

	/**
	 * 
	 */
	public void backLastDrillDown() {
		undoDrillDown(false);
	}

	/**
	 * 
	 */
	public void resetDrillDown() {
		undoDrillDown(true);
	}

	/**
	 * 
	 */
	public void filterIn(ArrayList<GraphNode> graphNodes) {
		filterGraphNodeInOrOut(graphNodes, true, null);
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setFilteringChanged(true);
		updateSDBackground(update);
	}

	/*
	 * Tree of collapsed lifelines: Horizontal collapsing
	 */
	private static final String HORIZONTAL_COLLAPSING = "horizontalCollapsing"; //$NON-NLS-1$
	private HashMap<Object, List<Object>> collapsedLifelines; // value must be List<Object>

	/**
	 * 
	 */
	public void collapseLifelines(List<GraphNode> graphNodes) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("collapseLifelines("+(graphNodes!=null ? graphNodes.size() : 0)+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (collapsedLifelines == null) {
			collapsedLifelines = new HashMap<Object, List<Object>>();
		}
		List<Object> list = new ArrayList<Object>();
		for (Iterator<GraphNode> i = graphNodes.iterator(); i.hasNext(); ) {
			GraphNode gn = i.next();
			Object key = null;
			if (gn instanceof TraceCollapsedLifelines) {
				key = ((TraceCollapsedLifelines)gn).getCollapsedLifelines();
			} else if (gn instanceof TraceLifeline) {
				key = getLifeLineId(((TraceLifeline)gn).getEObject());
			} else {
				continue; // Error ignored
			}
			list.add(key);
			collapsedLifelines.put(key, list);
		}
		saveCollapsedLifelines();
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setHorizontalCollapsingChanged(true);
		update.setVisibleX(frame.getVisibleAreaX());
		updateSDBackground(update);
	}

	/**
	 * 
	 */
	public void expandLifelines(List<GraphNode> graphNodes) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("expandLifelines("+(graphNodes!=null ? graphNodes.size() : 0)+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		int index = Integer.MAX_VALUE;
		for (Iterator<GraphNode> i = graphNodes.iterator(); i.hasNext(); ) {
			GraphNode gn = i.next();
			if (gn instanceof TraceCollapsedLifelines) {
				TraceCollapsedLifelines traceCollapsedLifelines = ((TraceCollapsedLifelines)gn);
				if (index > traceCollapsedLifelines.getIndex()) {
					index = traceCollapsedLifelines.getIndex();
				}
				List<Object> sublist = traceCollapsedLifelines.getCollapsedLifelines();
				for (Iterator<Object> j = sublist.iterator(); j.hasNext(); ) {
					Object key = j.next();
					collapsedLifelines.put(key, null);
				}
			} else  {
				continue; // Error ignored
			}
		}
		saveCollapsedLifelines();
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setHorizontalCollapsingChanged(true);
		update.setVisibleX(frame.getVisibleAreaX());
		updateSDBackground(update);
	}
	
	/**
	 * Implementation of ISDCollapseProvider
	 */
	public void collapseTwoLifelines(Lifeline lifeline1, Lifeline lifeline2) {
		List<GraphNode> list = new ArrayList<GraphNode>();
		// Reverse order for some usability reason
		list.add(lifeline2);
		list.add(lifeline1);
		collapseLifelines(list);
	}
	
	@SuppressWarnings("unchecked")
	private String parseHorizontalCollapsingTree(Object o) {
		String result = null;
		if (o instanceof List) {
			List<Object> list = (List<Object>)o;
			result = "{"; //$NON-NLS-1$
			String prefix = ""; //$NON-NLS-1$
			for (Iterator<Object> i = list.iterator(); i.hasNext(); ) {
				Object u = i.next();
				String parsingResult = parseHorizontalCollapsingTree(u);
				if (parsingResult != null) {
					result += prefix+parsingResult;
					prefix = ","; //$NON-NLS-1$
				}
			}
			result += "}"; //$NON-NLS-1$
		} else if (o instanceof String) {
			result = (String)o;
		}
		return result;
	}

	private void saveCollapsedLifelines() {
		String result = ""; //$NON-NLS-1$
		if (collapsedLifelines != null) {
			Set<Entry<Object, List<Object>>> entries = collapsedLifelines.entrySet();
			List<Object> uniqueTopLists = new ArrayList<Object>();
			for (Iterator<Entry<Object, List<Object>>> i = entries.iterator(); i.hasNext(); ) {
				Entry<Object, List<Object>> e = i.next();
				Object value = e.getValue();
				if (collapsedLifelines.get(value) == null &&
					!uniqueTopLists.contains(value)) {
					uniqueTopLists.add(value);
				}
			}
			String prefix = "{"; //$NON-NLS-1$
			for (Iterator<Object> i = uniqueTopLists.iterator(); i.hasNext(); ) {
				String parsingResult = parseHorizontalCollapsingTree(i.next());
				if (parsingResult != null) {
					result += prefix+parsingResult;
					prefix = ","; //$NON-NLS-1$
				}
			}
		}
		if (result.length() > 0) {
			result += "}"; //$NON-NLS-1$
		}
		DialogSettings settings = (DialogSettings)TraceUIPlugin.getDefault().getDialogSettings();
		settings.put(HORIZONTAL_COLLAPSING, result);
		TraceSDUtil.debugUml2SDTraceEvents("Saving "+HORIZONTAL_COLLAPSING+"="+result); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	private List<String> splitOneLevel(String input) {
		List<String> list = new ArrayList<String>();
		int level = 0, start = 1;
		int end = input.length()-1;
		for (int i = 1; i < end; i++) {
			char c = input.charAt(i);
			if (c == '{') {
				level++;
			} else if (c == '}') {
				level--;
			} else if ((c == ',' || i == end-1) && level == 0) {
				String member = input.substring(start, c==','?i:i+1);
				list.add(member);
				start = i+1;
			}
		}
		return list;
	}
	
	private Object parseHorizontalCollapsingString(String input, boolean deep) {
		if (input != null &&
			input.length() > 2 &&
			input.charAt(0) == '{' &&
			input.charAt(input.length()-1) == '}') {
			List<String> members = splitOneLevel(input);
			if (members != null && members.size() > 0) {
				List<Object> list = new ArrayList<Object>();
				for (Iterator<String> i = members.iterator(); i.hasNext(); ) {
					Object o = parseHorizontalCollapsingString(i.next(), true);
					if (o != null && deep) {
						list.add(o);
						collapsedLifelines.put(o, list);
					}
				}
				if (list.size() > 0){
					return list;
				}
			}
		} else if (input != null &&
				   input.length() > 0) {
			String parts[] = input.split("#"); //$NON-NLS-1$
			if (parts.length != 2 || TIUtils.doesAgentProxyStillExists(parts[0])) {
				return input; // Already a string atom
			} else {
				return null;
			}
		}
		return null;
	}
	
	private HashMap<Object, List<Object>> loadCollapsedLifelines() {
		DialogSettings settings = (DialogSettings)TraceUIPlugin.getDefault().getDialogSettings();
		String result = settings.get(HORIZONTAL_COLLAPSING);
		collapsedLifelines = new HashMap<Object, List<Object>>();
		parseHorizontalCollapsingString(result, false/*first level is not a lifeline, just a list*/);
		return collapsedLifelines;
	}
	
	/**
	 * Set of collapsed messages: vertical collapsing
	 */
	private HashMap<Object, HashSet<Object>> collapsedMessages;
	
	private static final String VERTICAL_COLLAPSING = "verticalCollapsing"; //$NON-NLS-1$
	
	private Object getMethodInvocationContainerKey(TRCFullMethodInvocation methodInvocation) {
		return methodInvocation.getProcess().getAgent().getAgentProxy().getRuntimeId();
	}
	
	private Object getMethodInvocationCollapsingId(TRCFullMethodInvocation methodInvocation) {
		return methodInvocation.getThread().getId()+":"+methodInvocation.getTicket(); //$NON-NLS-1$
	}
	
	private HashSet<Object> getContainerKeyHashSet(TRCFullMethodInvocation methodInvocation) {
		if (collapsedMessages == null) {
			return null;
		}
		HashSet<Object> object = collapsedMessages.get(getMethodInvocationContainerKey(methodInvocation));
		return object != null ? (HashSet<Object>)object : null;
	}

	private void saveCollapsedMessages() {
		StringBuffer result = new StringBuffer(); //$NON-NLS-1$
		if (collapsedMessages != null) {
			Set<Object> keys = collapsedMessages.keySet();
			for (Iterator<Object> i = keys.iterator(); i.hasNext(); ) {
				String key = (String)i.next();
				HashSet<Object> containerSet = collapsedMessages.get(key);
				if (containerSet.size() > 0) {
					result.append(key).append("="); //$NON-NLS-1$
					String value;
					for (Iterator<Object> j = containerSet.iterator(); j.hasNext(); ) {
						value = (String)j.next();
						//this is too expensive; commenting for now
						//TraceSDUtil.debugUml2SDTraceEvents("Save: key="+key+" ticket="+value); //$NON-NLS-1$ //$NON-NLS-2$
						result.append(value).append((j.hasNext()?";":"")); //$NON-NLS-1$ //$NON-NLS-2$
					}
					result.append("]"); //$NON-NLS-1$
				}
			}
		}
		DialogSettings settings = (DialogSettings)TraceUIPlugin.getDefault().getDialogSettings();
		settings.put(VERTICAL_COLLAPSING, result.toString());
	}

	private HashMap<Object, HashSet<Object>> loadCollapsedMessages() {
		DialogSettings settings = (DialogSettings)TraceUIPlugin.getDefault().getDialogSettings();
		HashMap<Object, HashSet<Object>> ret = null;
		String result = settings.get(VERTICAL_COLLAPSING);
		if (result != null && result.length() > 1) {
			String agents[] = result.split("]"); //$NON-NLS-1$
			for (int i = 0; i < agents.length; i++) {
				String parts[] = agents[i].split("="); //$NON-NLS-1$
				if (TIUtils.doesAgentProxyStillExists(parts[0])) {
					HashSet<Object> containerSet;
					String values[] = parts[1].split(";"); //$NON-NLS-1$
					if (values.length > 0) {
						if (ret == null) {
							ret = new HashMap<Object, HashSet<Object>>();
						}
						ret.put(parts[0], containerSet = new HashSet<Object>());
						for (int j = 0; j < values.length; j++) {
							containerSet.add(values[j]);
							TraceSDUtil.debugUml2SDTraceEvents("Load: key="+parts[0]+" ticket="+values[j]); //$NON-NLS-1$ //$NON-NLS-2$
						}
					}
				} else {
					TraceSDUtil.debugUml2SDTraceEvents("Agent proxy "+parts[0]+" does no longer exist"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
		return ret;
	}

	/**
	 * @param methodInvocation
	 * @return
	 */
	private boolean isMessageCollapsed(TRCFullMethodInvocation methodInvocation) {
		if (methodInvocation == null || collapsedMessages == null) {
			return false;
		}
		HashSet<Object> containerSet = getContainerKeyHashSet(methodInvocation);
		if (containerSet != null &&
			containerSet.contains(getMethodInvocationCollapsingId(methodInvocation))) {
			return true;
		}
		for (TRCFullMethodInvocation invokedBy = (TRCFullMethodInvocation)methodInvocation.getInvokedBy();
			 invokedBy != null;
			 invokedBy = (TRCFullMethodInvocation)invokedBy.getInvokedBy()) {
			containerSet = getContainerKeyHashSet(invokedBy);
			if (containerSet != null &&
				containerSet.contains(getMethodInvocationCollapsingId(invokedBy))) {
				return true;
			}
		}
		return false;
	}

	/**
	 * 
	 */
	public void collapseCalledMessages(ArrayList<GraphNode> graphNodes) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("collapseCalledMessages("+(graphNodes!=null ? graphNodes.size() : 0)+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		for (Iterator<GraphNode> i = graphNodes.iterator(); i.hasNext(); ) {
			Object gn = i.next();
			TRCFullMethodInvocation methodInvocation = null;
			if (gn instanceof TraceSyncMessage) {
				TraceSyncMessage sm = (TraceSyncMessage)gn;
				methodInvocation = (TRCFullMethodInvocation)sm.getEObject();
			} else if (gn instanceof TraceSyncMessageReturn) {
				TraceSyncMessageReturn smr = (TraceSyncMessageReturn)gn;
				methodInvocation = (TRCFullMethodInvocation)smr.getEObject();
			} // else ignored
			collapseCalledMessage(methodInvocation);
		}
		saveCollapsedMessages();
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setVerticalCollapsingChanged(true);
		updateSDBackground(update);
	}
	
	/**
	 * @param methodInvocation
	 */
	private void collapseCalledMessage(TRCFullMethodInvocation methodInvocation) {
		if (methodInvocation != null) {
			if (collapsedMessages == null) {
				collapsedMessages = new HashMap<Object, HashSet<Object>>();
			}
			HashSet<Object> containerSet = getContainerKeyHashSet(methodInvocation);
			if (containerSet == null) {
				collapsedMessages.put(getMethodInvocationContainerKey(methodInvocation), containerSet = new HashSet<Object>());
			}
			containerSet.add(getMethodInvocationCollapsingId(methodInvocation));
		}
	}

	/**
	 * 
	 */
	public void expandCalledMessages(ArrayList<GraphNode> graphNodes) {
		if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("expandCalledMessages("+(graphNodes!=null ? graphNodes.size() : 0)+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		for (Iterator<GraphNode> i = graphNodes.iterator(); i.hasNext(); ) {
			Object gn = i.next();
			TRCFullMethodInvocation methodInvocation = null;
			if (gn instanceof TraceSyncMessage) {
				TraceSyncMessage sm = (TraceSyncMessage)gn;
				methodInvocation = (TRCFullMethodInvocation)sm.getEObject();
			} else if (gn instanceof TraceSyncMessageReturn) {
				TraceSyncMessageReturn smr = (TraceSyncMessageReturn)gn;
				methodInvocation = (TRCFullMethodInvocation)smr.getEObject();
			} // else ignored
			if (methodInvocation != null) {
				HashSet<Object> containerSet = getContainerKeyHashSet(methodInvocation);
				if (containerSet == null) {
					continue; // strange, but...
				}
				containerSet.remove(getMethodInvocationCollapsingId(methodInvocation));
			}
		}
		saveCollapsedMessages();
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setVerticalCollapsingChanged(true);
		updateSDBackground(update);
	}

	/**
	 * Highlight call stack action
	 */
	public void highlightCallStack(ArrayList<GraphNode> graphNodes) {
		if (graphNodes.get(0) instanceof ITraceMessage) {
			// It's an IEObjectSelection
			IEObjectSelection eos = (IEObjectSelection)graphNodes.get(0);
			EObject eo = eos.getEObject();
			if (eo instanceof TRCFullMethodInvocation) {
				TRCFullMethodInvocation mi = (TRCFullMethodInvocation)eo;
				LinkedList<TRCFullMethodInvocation> callStack = new LinkedList<TRCFullMethodInvocation>();
				callStack.add(mi);
				for (mi = (TRCFullMethodInvocation)mi.getInvokedBy();
					 mi != null;
					 mi = (TRCFullMethodInvocation)mi.getInvokedBy()) {
					callStack.addFirst(mi);
				}
				Iterator<TRCFullMethodInvocation> it = callStack.iterator();
				EObject eo1 = it.next();
				List<GraphNode> newSelection = new ArrayList<GraphNode>();
				for (int i = 0; eo1 != null && i < frame.syncMessageCount(); i++) {
					EObject eo2 = ((IEObjectSelection)frame.getSyncMessage(i)).getEObject();
					if (eo1 == eo2) {
						newSelection.add(frame.getSyncMessage(i));
						if (it.hasNext()) {
							eo1 = it.next();
						} else {
							eo1 = null;
						}
					}
				}
				view.getSDWidget().clearSelection();
				view.getSDWidget().addSelection(newSelection);
				view.getSDWidget().ensureVisible(newSelection.get(newSelection.size()-1));
				view.getSDWidget().redraw();
			}
		}
	}
	
	/**
	 * Lifeline reordering attribute
	 */
	private ArrayList<Lifeline[]> lifelineReordered = null;

	/**
	 * Tries to reorder the lifelines after an update
	 */
	private void reorderLifelinesAgain() {
		if (view != null && view.getSDWidget() != null) {
			ArrayList<Lifeline[]> lastLifelineReordered = view.getSDWidget().getLifelineReoderList();
			if (lastLifelineReordered != null) {
				if (lifelineReordered == null) {
					lifelineReordered = new ArrayList<Lifeline[]>();
				}
				lifelineReordered.addAll(lastLifelineReordered);
			}
		}
		if (lifelineReordered == null || lifelineReordered.size() == 0 || frame == null) {
			return;
		}
		ArrayList<Lifeline[]> newList = new ArrayList<Lifeline[]>();
		for (Iterator<Lifeline[]> i = lifelineReordered.iterator(); i.hasNext(); ) {
			Lifeline op[] = i.next();
			if (op.length == 2) {
				Lifeline nop[] = new Lifeline[2];
				nop[0] = getNewerLifeline(op[0]);
				nop[1] = getNewerLifeline(op[1]);
				if (nop[0] != null && nop[1] != null) {
					newList.add(nop);
				}
			}
		}
		frame.reorder(newList);
		lifelineReordered = newList;
	}

	/**
	 * @param op
	 */
	private Lifeline getNewerLifeline(Lifeline op) {
		try {
			EObject eObject = ((TraceLifeline)op).getEObject();
			if (eObject != null) {
				TraceLifelineDraft traceLifelineDraft = instancesMap.get(getLifeLineId(eObject));
				if (traceLifelineDraft != null) {
					return traceLifelineDraft.getLifeline();
				}
			}
		} catch (Exception e) {
			if (TraceSDUtil.debugLoader) {
				e.printStackTrace();
			}
		}
		return null;
	}

	/**
	 * Implementation of ITraceInteractionPreferenceListener
	 */
	public void applyPreferences() {
		int max = (int)TraceUIPlugin.getDefault().getPreferenceStore().
							getLong(ITraceInteractionPreferenceListener.PAGE_SIZE);
		if (maximumMessagesByPage == max) {
			return;
		}
		TraceInteractionUpdate update = new TraceInteractionUpdate();
		update.setMaxMessagesByPageChanged(true);
		maximumMessagesByPage = max;
		updateSDBackground(update);
	}

	/**
	 * Extension of ISDExtendedActionBarProvider
	 */
	public void supplementCoolbarContent(IActionBars bar) {
		super.supplementCoolbarContent(bar);

		MenuManager changeTraceKind = new MenuManager(TraceUIMessages._138);
		bar.getMenuManager().appendToGroup("UML2SD_VIEW_MODES", changeTraceKind); //$NON-NLS-1$
		List<ChangeLifelineLoaderAction> list = ChangeLifelineLoaderAction.getChangeLifelineLoaderActions(view.getViewSite().getId());
		for (Iterator<ChangeLifelineLoaderAction> i = list.iterator(); i.hasNext(); ) {
			ChangeLifelineLoaderAction a = i.next();
			a.setEnabled(!a.getLoaderClassName().equals(lifelineLoader.getClass().getName()));
			changeTraceKind.add(a);
		}

		undoDrillDownMenuAction = new UndoDrillDownMenuAction();
		bar.getMenuManager().appendToGroup("UML2SD_FILTERING", undoDrillDownMenuAction.getBackLastDrillDownAction()); //$NON-NLS-1$
		bar.getMenuManager().appendToGroup("UML2SD_FILTERING", undoDrillDownMenuAction.getResetDrillDownAction()); //$NON-NLS-1$
		bar.getToolBarManager().appendToGroup("UML2SD_FILTERING", undoDrillDownMenuAction); //$NON-NLS-1$
		undoDrillDownMenuAction.setEnabled(hasPositiveFilters);
		
	}
	

	private static TraceInteractions instance;

	/**
	 * returns the firstly created instance of TraceInteractions
	 */
	public static TraceInteractions getInstance() {
		if (instance == null) {
			instance = new TraceInteractions();
		}
		return instance;
	}
	
	private ILifelineLoader lifelineLoader;

	/**
	 * @param lifelineLoader The lifelineLoader to set.
	 */
	public void setLifelineLoader(String className) {
		lifelineLoader = (ILifelineLoader)LoadersManager.getLoadersManager().createLoaderInstance(className, getClass().getClassLoader());
		if (view != null) {
			TraceInteractionUpdate update = new TraceInteractionUpdate();
			update.setLifelineLoaderChanged(true);
			updateSDBackground(update);
		} else {
			LoadersManager.getLoadersManager().createLoader(className, getClass().getClassLoader(), view);
			// This has just caused setViewer to be called, the view to be set and the model to be read
		}
		SDUtil.getInstance().setLastLoader(className, view.getSite().getId());
	}

	/**
	 * @param loader
	 */
	public void setLifelineLoader(BaseLifelineLoader lifelineLoader_) {
		lifelineLoader = lifelineLoader_;
	}

	/**
	 * @return Returns the lifelineLoader.
	 */
	public ILifelineLoader getLifelineLoader() {
		return lifelineLoader;
	}

	/**
	 * @see org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions#getFinalTitleString()
	 */
	public String getFinalTitleString() {
		if (lifelineLoader != null) {
			return lifelineLoader.getFinalTitleString();
		}
		return null;
	}

	/**
	 * @see org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions#getLifelineEObjectFromMethodInvocation(org.eclipse.hyades.models.trace.TRCFullMethodInvocation)
	 */
	protected EObject getLifelineEObjectFromMethodInvocation(TRCFullMethodInvocation mi) {
		if (lifelineLoader != null) {
			return lifelineLoader.getLifelineEObjectFromMethodInvocation(mi);
		}
		return null;
	}

	/**
	 * @see org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions#getLifeLineTitle(org.eclipse.emf.ecore.EObject, boolean)
	 */
	protected String getLifeLineTitle(EObject to, boolean long_) {
		if (lifelineLoader != null) {
			return lifelineLoader.getLifeLineTitle(to, long_);
		}
		return null;
	}

	protected String getDistinguishedLifeLineTitle(EObject to, int rank) {
		if (lifelineLoader != null) {
			return lifelineLoader.getDistinguishedLifeLineTitle(to, rank);
		}
		return null;
	}

	/**
	 * @see org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions#getLifeLineTitle(org.eclipse.emf.ecore.EObject, boolean)
	 */
	protected String getLifeLineId(EObject to) {
		if (lifelineLoader != null) {
			return lifelineLoader.getLifeLineId(to);
		}
		return null;
	}

	/**
	 * @see org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions#getLifeLineCategory(org.eclipse.emf.ecore.EObject)
	 */
	protected int getLifeLineCategory(EObject to) {
		if (lifelineLoader != null) {
			return lifelineLoader.getLifeLineCategory(to);
		}
		return 0;
	}

	public String getNodeName(int nodeType, String loaderClassName) {
		ILifelineLoader loader;
		if (loaderClassName == null) {
			// Use the current lifeline loader
			loader = lifelineLoader;
			
		} else {
			loader = (ILifelineLoader)LoadersManager.getLoadersManager().createLoaderInstance(loaderClassName, getClass().getClassLoader());
		}
		switch (nodeType) {
			case ISDGraphNodeSupporter.LIFELINE:
				return loader.getLifelineTypeName();
			case ISDGraphNodeSupporter.SYNCMESSAGE:
				return loader.getMessageTypeName();
			default:
				return ""; //$NON-NLS-1$
		}
	}

	/**
	 * Implementation of ISDPropertiesProvider
	 */
	public IPropertySheetPage getPropertySheetEntry() {
		PropertySheetPage pse = new PropertySheetPage();
		pse.setRootEntry(new TraceRootProperty());
		return pse;
	}

}
